// -*- C++ -*-
/*!
 * @file  DFIT.cpp
 * @brief Dual Floor Image Tracking
 * @date $Date$
 *
 * @author 芝浦工業大学 水川研究室
 * 〒135-8548 東京都江東区豊洲3-7-5芝浦工業大学 研究棟11Q32 水川研究室 TEL：03-5859
 * -8209 FAX：03-5859-8201
 * Mail：shibaura.hri.goiken@gmail.com
 * 指導教員：水川 真 作成者：鷹栖 尭大
 * URL：http://www.hri.ee.shibaura-it.ac.jp/
 *
 * 日立情報制御ソリューションズ社製のIP7000BDを制御するソースコード及びライブラリ
 * を除き，DFITコンポーネントの著作権は，芝浦工業大学水川研究室に帰属します． 
 * IP7000BDを制御するソースコード及びライブラリの著作権は，開発元の”日立情報制御ソ
 * リューションズ”にあります．
 *
 * $Id$
 */

#include "DFIT.h"

// Module specification
// <rtc-template block="module_spec">
static const char* dfit_spec[] =
  {
    "implementation_id", "DFIT",
    "type_name",         "DFIT",
    "description",       "Dual Floor Image Tracking",
    "version",           "1.0.0",
    "vendor",            "Shibaura Institute of Technology",
    "category",          "Camera",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.FileName", "DFITlog.csv",
    "conf.default.CameraDistance", "575",
    "conf.default.MPPR", "0.328125",
    "conf.default.MPPL", "0.328125",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
DFIT::DFIT(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_position_outOut("Position_Out", m_position_out),
	m_pDFIT(NULL)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
DFIT::~DFIT()
{
}


/*!
 * 計測用クラスのインスタンスを生成し，画像処理ボードの初期化設定を行う．
 */
RTC::ReturnCode_t DFIT::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("Position_Out", m_position_outOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("FileName", m_FileName, "DFITlog.csv");
  bindParameter("CameraDistance", m_CameraDistance, "575");
  bindParameter("MPPR", m_MPPR, "0.328125");
  bindParameter("MPPL", m_MPPL, "0.328125");

	m_pDFIT = new DFIT_Cal();

  // </rtc-template>
  return RTC::RTC_OK;
}

/*!
 * 計測クラスのインスタンスを破棄する．
 */

RTC::ReturnCode_t DFIT::onFinalize()
{
	if(m_pDFIT != NULL)
	{
		delete m_pDFIT;
		m_pDFIT == NULL;
	}
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t DFIT::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DFIT::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * コンポーネントのConfigurationを適用する．
 */

RTC::ReturnCode_t DFIT::onActivated(RTC::UniqueId ec_id)
{
	m_configsets.update();
	m_position_out.data.length(3);
	m_pDFIT->Configuration(m_FileName, m_CameraDistance, m_MPPR, m_MPPL);
	m_pDFIT->TPmatching();

  return RTC::RTC_OK;
}

/*!
 * 計測を一時停止する．
 */

RTC::ReturnCode_t DFIT::onDeactivated(RTC::UniqueId ec_id)
{
	m_pDFIT->Stop();
  return RTC::RTC_OK;
}

/*!
 * カメラの画像処理データよりロボットの自己位置を計算し，アウトポートから出力する
 * ．
 */

RTC::ReturnCode_t DFIT::onExecute(RTC::UniqueId ec_id)
{
	double x, y, theta, timestamp;

	m_pDFIT->GetPosition(x, y, theta, timestamp);

	m_position_out.data[0] = x;
	m_position_out.data[1] = y;
	m_position_out.data[2] = theta;
	m_position_out.tm.nsec = timestamp;

	m_position_outOut.write(m_position_out);
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t DFIT::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DFIT::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * 自己位置をリセットする．
 */

RTC::ReturnCode_t DFIT::onReset(RTC::UniqueId ec_id)
{
	m_pDFIT->ResetPosition();
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t DFIT::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DFIT::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void DFITInit(RTC::Manager* manager)
  {
    coil::Properties profile(dfit_spec);
    manager->registerFactory(profile,
                             RTC::Create<DFIT>,
                             RTC::Delete<DFIT>);
  }
  
};


