// -*- C++ -*-
/*!
 * @file  DFIT.h
 * @brief Dual Floor Image Tracking
 * @date  $Date$
 *
 * @author 芝浦工業大学 水川研究室
 * 〒135-8548 東京都江東区豊洲3-7-5芝浦工業大学 研究棟11Q32 水川研究室 TEL：03-5859
 * -8209 FAX：03-5859-8201
 * Mail：shibaura.hri.goiken@gmail.com
 * 指導教員：水川 真 作成者：鷹栖 尭大
 * URL：http://www.hri.ee.shibaura-it.ac.jp/
 *
 * 日立情報制御ソリューションズ社製のIP7000BDを制御するソースコード及びライブラリ
 * を除き，DFITコンポーネントの著作権は，芝浦工業大学水川研究室に帰属します． 
 * IP7000BDを制御するソースコード及びライブラリの著作権は，開発元の”日立情報制御ソ
 * リューションズ”にあります．
 *
 * $Id$
 */

#ifndef DFIT_H
#define DFIT_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

#include "DFIT_Cal.h"

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

/*!
 * @class DFIT
 * @brief Dual Floor Image Tracking
 *
 * カメラより得られた路面画像より，ロボットの移動距離を算出，その積分よりロボット
 * の自己位置を求める．
 *
 * ロボットの自己位置（x[m],y[m],theta[deg])
 *
 */
class DFIT
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  DFIT(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~DFIT();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * 計測用クラスのインスタンスを生成し，画像処理ボードの初期化設定を行う．
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre XC-HC50が2つ接続された画像処理ボードIP7000BDがRTC実行PCに接続されている
   * こと．
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * 計測クラスのインスタンスを破棄する．
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * コンポーネントのConfigurationを適用する．
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre Configurationが設定されていること．
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   * 計測を一時停止する．
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * カメラの画像処理データよりロボットの自己位置を計算し，アウトポートから出力す
   * る．
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   * 自己位置をリセットする．
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * ログファイル名を設定する．
   * - Name: FileName FileName
   * - DefaultValue: DFITlog.csv
   * - Constraint: 拡張子".csv"を含む文字列であること．
   */
  std::string m_FileName;
  /*!
   * 2つのカメラ間の距離を設定する．
   * - Name: CameraDistance CameraDistance
   * - DefaultValue: 575
   * - Unit: mm
   */
  int m_CameraDistance;
  /*!
   * 右のカメラの1ピクセルあたりの距離を設定する．
   * - Name: MPPR MPPR
   * - DefaultValue: 0.328125
   * - Unit: mm/pixel
   */
  double m_MPPR;
  /*!
   * 左のカメラの1ピクセルあたりの距離を設定する．
   * - Name: MPPL MPPL
   * - DefaultValue: 0.328125
   * - Unit: mm/pixel
   */
  double m_MPPL;
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_position_out;
  /*!
   * カメラより得られた路面画像より，ロボットの移動距離を算出，その積分より求めた
   * 自己位置を出力する．
   * - Type: TimedDoubleSeq
   * - Number: 3
   * - Semantics: 配列番号[0]：ロボットのX軸方向自己位置[m]
   *              配列番号[1]：ロボットのY軸方向自己位置[m]
   *              配列番号[2]：ロボットの向き[deg]
   * - Unit: (m,m,deg)
   * - Frequency: rtc.confの設定に従う．
   */
  OutPort<TimedDoubleSeq> m_position_outOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>
	 DFIT_Cal* m_pDFIT;

};


extern "C"
{
  DLL_EXPORT void DFITInit(RTC::Manager* manager);
};

#endif // DFIT_H
